#include <stdio.h>

#include <itkImage.h>
#include <itkVector.h>
#include <itkImageFileReader.h>
#include <itkImageFileWriter.h>
#include <itkExtractImageFilter.h>

#define itkNew(type,name) \
		type::Pointer name = type::New();

template<class ImageType>
void validate(const char *fname, unsigned int n) {
	typedef itk::ImageFileReader<ImageType> ReaderType;
	typedef itk::ExtractImageFilter<ImageType, ImageType> ExtractFilterType;

	typename ImageType::RegionType roi;
	roi.SetSize(0, 1);
	roi.SetSize(1, 1);
	roi.SetSize(2, n);

	typename itkNew(ReaderType, reader);
	reader->SetFileName(fname);

	typename itkNew(ExtractFilterType, extractor);
	extractor->SetInput(reader->GetOutput());
	extractor->SetExtractionRegion(roi);
	extractor->InPlaceOn();
	extractor->Update();

	typename ImageType::IndexType idx = { { 0, 0, 0 } };
	for (unsigned int i = 0; i < n; ++i) {
		idx[2] = i;
		std::cout << i << ":\t" << extractor->GetOutput()->GetPixel(idx) << std::endl;
	}
	std::cout << std::endl;
}

int main(int argc, char **argv) {
	const char *fname = "./img.hdf5";

    typedef double PixelType;
	// typedef itk::Vector<double, 3> PixelType;
    // same result but warning messages due to poor pixel type handling in metaIO

	typedef itk::Image<PixelType, 3> ImageType;

	itkNew(ImageType, img);
	{ // init image image
		ImageType::RegionType region;
		region.SetSize(0, 16);
		region.SetSize(1, 32);
		region.SetSize(2, 64);
		img->SetRegions(region);
		img->Allocate();
		img->FillBuffer(PixelType(1.0));
	}

	{ // write entire image
		typedef itk::ImageFileWriter<ImageType> WriterType;
		itkNew(WriterType, writer);
		writer->SetInput(img);
		writer->SetFileName(fname);
		writer->Write();
	}

	std::cout << "before paste:" << std::endl;
	validate<ImageType>(fname, 10);

	itkNew(ImageType, pasteImg);
	{	// init single slice to be pasted into existing file
		ImageType::RegionType pasteRegion = img->GetLargestPossibleRegion();
		pasteRegion.SetIndex(2, 5);
		pasteRegion.SetSize(2, 1);
		pasteImg->SetRegions(img->GetLargestPossibleRegion());
		pasteImg->SetBufferedRegion(pasteRegion);
		// pasteImg->SetRegions(pasteRegion);
		pasteImg->Allocate();
		pasteImg->FillBuffer(PixelType(2.0));
	}

	{ // paste image
		itk::ImageIORegion ioRegion(3);
		for (unsigned int i = 0; i < 3; ++i) {
			ioRegion.SetIndex(i, pasteImg->GetBufferedRegion().GetIndex(i));
			ioRegion.SetSize(i, pasteImg->GetBufferedRegion().GetSize(i));
		}

		typedef itk::ImageFileWriter<ImageType> WriterType;
		itkNew(WriterType, writer);
		writer->SetInput(pasteImg);
		writer->SetFileName(fname);
		writer->SetIORegion(ioRegion);
		writer->Write();
	}

	std::cout << "after paste:" << std::endl;
	validate<ImageType>(fname, 10);

	return EXIT_SUCCESS;
}

